@@ -1,6 +1,4 @@
 package Catmandu::Importer::RDF;
-#ABSTRACT: parse RDF data
-our $VERSION = '0.16'; #VERSION
 
 use namespace::clean;
 use Catmandu::Sane;
@@ -8,8 +6,11 @@ use Moo;
 use RDF::Trine::Parser;
 use RDF::Trine::Model;
 use RDF::aREF;
+use RDF::aREF::Encoder;
 use RDF::NS;
 
+our $VERSION = '0.16';
+
 with 'Catmandu::RDF';
 with 'Catmandu::Importer';
 
@@ -17,14 +18,6 @@ has url => (
     is => 'ro'
 );
 
-has 'sn' => (
-    is => 'ro',
-    lazy    => 1, 
-    builder => sub {
-        $_[0]->ns ? $_[0]->ns->REVERSE : undef
-    }
-);
-
 has base => (
     is      => 'ro', 
     lazy    => 1, 
@@ -33,40 +26,13 @@ has base => (
     }
 );
 
-# TODO: move to RDF::aREF
-sub uri2aref {
-    my ($self, $uri, $sep) = @_;
-
-    return 'a' if $uri eq 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type';
-
-    if ($self->sn) {
-        my @qname = $self->sn->qname($uri);
-        return join($sep,@qname) if @qname;
+has encoder => (
+    is      => 'ro',
+    lazy    => 1,
+    builder => sub {
+        RDF::aREF::Encoder->new( ns => $_[0]->ns );
     }
-
-    return $uri;
-}
-# TODO: move to RDF::aREF
-sub rdfjson2aref {
-    my ($self, $object) = @_;
-    if ($object->{type} eq 'literal') {
-        my $value = $object->{value};
-        if ($object->{lang}) {
-            return $value.'@'.$object->{lang};
-        } elsif ($object->{datatype}) {
-            my $dt = $self->uri2aref($object->{datatype},':');
-            $dt = "<$dt>" if $dt eq $object->{datatype};
-            return "$value^$dt";
-        } else {
-            return "$value@";
-        }
-    } elsif ($object->{type} eq 'bnode') {
-        return $object->{value};
-    } else {
-        my $obj = $self->uri2aref($object->{value},':');
-        return ($obj eq $object->{value} ? "<$obj>" : $obj);
-    }
-}
+);
 
 sub generator {
     my ($self) =@_;
@@ -80,14 +46,19 @@ sub generator {
         # TODO: include namespace mappings if requested
         while (my ($s,$ps) = each %$stream) {
             foreach my $p (keys %$ps) {
-                my $predicate = $self->uri2aref($p,'_');
+                my $predicate = $self->encoder->predicate($p);
                $stream->{$s}->{$predicate} = [
-                    map { $self->rdfjson2aref($_) } @{$stream->{$s}->{$p}} 
+                    map { $self->encoder->object($_) } @{$stream->{$s}->{$p}} 
                 ]; 
                 delete $stream->{$s}->{$p} if $predicate ne $p;
             }
         }
         $aref = $stream;
+
+        if ($self->url) {
+            $aref->{_url} = $self->url;
+        }
+
         $stream = undef;
 
         return $aref;
@@ -113,30 +84,25 @@ sub _rdf_stream {
     return $model->as_stream;
 }
 
-
 1;
-
 __END__
 
-=pod
-
-=encoding UTF-8
-
 =head1 NAME
 
 Catmandu::Importer::RDF - parse RDF data
 
-=head1 VERSION
-
-version 0.16
-
 =head1 SYNOPSIS
 
-  catmandu convert RDF --file rdfdump.ttl to YAML
+Command line client C<catmandu>:
 
-=head1 SYNOPSIS
+    catmandu convert RDF --url http://d-nb.info/1001703464 to YAML
+    catmandu convert RDF --file rdfdump.ttl to JSON
 
-  catmandu convert RDF --url http://d-nb.info/1001703464 to YAML
+In Perl code:
+
+    use Catmandu::Importer::RDF;
+    my $url = "http://dx.doi.org/10.2474/trol.7.147";
+    my $rdf = Catmandu::Importer::RDF->new( url => $url )->first;
 
 =head1 DESCRIPTION
 
@@ -180,19 +146,14 @@ Set to a specific date to get stable namespace prefix mappings.
 
 =back
 
+=head1 METHODS
+
+See L<Catmandu::Importer>.
+
 =head1 SEE ALSO
 
 L<RDF::Trine::Store>, L<RDF::Trine::Parsers>
 
-=head1 AUTHOR
-
-Jakob Voß
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2014 by Jakob Voß.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
+=encoding utf8
 
 =cut
